'' =================================================================================================
''
''   File....... jm_ws2801_demo.spin
''   Purpose.... High-performance driver for the WS2801
''   Author..... Jon "JonnyMac" McPhalen 
''               Copyright (c) 2012 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 25 JUL 2012
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
' _xinfreq = 6_250_000                                          ' use 6.25MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

con

  RX1     = 31                                                  ' programming / debug port
  TX1     = 30
  
  SDA     = 29                                                  ' eeprom
  SCL     = 28

  OUT_EN  = 27                                                  ' outputs enable pin (low)     
  
  SIO     = 26                                                  ' TTL serial io
  
  RX2     = 23                                                  ' RS-485 port
  RX2_EN  = 22
  TX2_EN  = 21
  TX2     = 20

  R_LED   = 18                                                  ' bi-color LED
  G_LED   = 17

  OUT7    = 15                                                  ' outputs
  OUT6    = 14   
  OUT5    = 13 
  OUT4    = 12
  OUT3    = 11
  OUT2    = 10   
  OUT1    =  9 
  OUT0    =  8
          
  OPT_BR  =  7                                                  ' options swithes
  OPT_A1  =  6   
  OPT_A0  =  5
  OPT_SM  =  4

  DMX_A8  =  3                                                  ' high-bit off DMX address

  DO_165  =  2                                                  ' control lines for 74x165
  CLK_165 =  1 
  LD_165  =  0


  ' WS2801 connections
  ' -- use TTL header
  
  CKI     =  OUT1
  SDI     =  OUT0


con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  leds : "jm_ws2801"
  rr   : "realrandom"
  prng : "jm_prng"
 

var

  long  target[8]                                               ' target for each channel
  long  level[8]                                                ' level of each channel
  long  flash[8]
  
  long  brightness            

  long  scog
  long  stack[32]


pub main | t, ch, bright, dither, color

  ' setup HC-8+ hardware

  outa[OUT7..OUT0] := %0000_0000                                ' clear outputs
  dira[OUT7..OUT0] := %1111_1111                                ' output mode                                  
  low(OUT_EN)                                                   ' enable 74x245
                                                    
  leds.start(SDI, CKI, 8, 2)                                    ' 8 channels, 2ms frame

  rr.start                                                      ' load hardware randomizer
  prng.seed(rr.random, rr.random, rr.random, rr.random, rr.random)  
  rr.stop                                                       ' unload hardware randomize

  
  ' basic color control demo

  t := cnt 
  repeat ch from 0 to 7
    repeat bright from 0 to 255                                 
      leds.set_red(ch, bright)                                  ' fade up red
      waitcnt(t += (MS_001 << 2))
   
  repeat ch from 0 to 7
    repeat bright from 0 to 255
      leds.set_green(ch, bright)                                ' fade up green
      leds.set_red(ch, 255-bright)                              ' fade down red
      waitcnt(t += (MS_001 << 2))     
   
  repeat ch from 0 to 7
    repeat bright from 0 to 255
      leds.set_blue(ch, bright)                                 ' fade up blue
      leds.set_green(ch, 255-bright)                            ' fade down green
      waitcnt(t += (MS_001 << 2))
   
  repeat bright from 255 to 0                                   ' fade out all
    leds.set_all(bright)
    waitcnt(t += (MS_001 << 3))
   
  pause(500)


  ' energy ball demo

  repeat ch from 0 to 7                                         ' seed the nodes
    target[ch] := (prng.random >> 1) // 125 + 76                ' set 1st target

  repeat
    brightness := brightness + 1 <# 255                         ' ramp up if down
    if (brightness == 255)                                      ' at full brightness?
      if (scog == 0)                                            ' if sparkler cog not running
        scog := cognew(sparkler, @stack) + 1                    '  start it
    
    repeat ch from 0 to 7
      if (level[ch] == target[ch])                              ' at target?            
        target[ch] := (prng.random >> 1) // 190 + 10            '  yes, set new one     
      elseif (level[ch] < target[ch])                           ' below target?         
        level[ch] := ++level[ch] <# 255                         '  increase and limit   
      else                                                      ' above target          
        level[ch] := --level[ch] #> 0                           '  decrease and limit
      
      if (flash[ch] <> $00_00_00)
        leds.set(ch, flash[ch])
      else
        leds.set(ch, level[ch] * brightness / 255)
        dither := prng.random >> (32 - 2) 
        leds.set_green(ch, dither)        
        leds.set_red(ch, dither)
        
    pause(5)


pri sparkler | playlist, last, ch, delay

'' Randomly flash 1 of 8 channels
'' -- no repeats until all flashed
'' -- do no flash same led twice in succession

  playlist := %00000000
  last := -1

  repeat
    repeat
      ch := prng.random & %111
      if (ch <> last)
        if ((playlist & (1 << ch)) == 0)
          playlist |= 1 << ch
          if (playlist == %11111111)
            playlist := %00000000
          last := ch
          quit

    flash[ch] := $FF_FF_FF
    delay := (prng.random >> 1) // 50 + 50
    pause(delay)
                            
    flash[ch] := $00_00_00
    delay := (prng.random >> 1) // 900 + 100
    pause(delay) 


con

  { ------------- }
  {  B A S I C S  } 
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1792                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]

          
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  